------------------------------------------------------------------------
--     BasysDemo.vhd -- Demonstrate basic Basys function
------------------------------------------------------------------------
-- Author:  Joshua Pederson
--          Copyright 2006 Digilent, Inc.
------------------------------------------------------------------------
-- This module tests basic device function and connectivity on the Basys
-- board. It was developed using the Xilinx WebPack tools.
--
--  Inputs:
--		mclk		- system clock (Oscillator on Basys board)
--		bn		- buttons on the Basys board
--		swt		- switches on the Basys board (8 switches)
--
--  Outputs:
--		led		- discrete LEDs on the Basys board (8 leds)
--		an		- anode lines for the 7-seg displays on Basys
--		ssg		- cathodes (segment lines) for the displays on Basys
--
--  VGA Components:
--		hs		- horizontal monitor sync
--		vs		- vertical monitor sync
--		red		- red pixel color
--		green		- green pixel color
--		blue		- blue pixel color
--
--  Connector Loopback Test Compontents:
--		LoopOut	- 6 pin header and PS2 out pins (JA pins, JC pins, PS2 data pin)
--		LoopIn	- 6 pin header and PS2 in pins (JB pins, JD pins, PS2 clock pin)
--
------------------------------------------------------------------------
-- Revision History:
--  03/08/2006(JPederson): created
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity BasysDemo is
    Port (
        mclk    : in std_logic;
        btn     : in std_logic_vector(3 downto 0);
        swt     : in std_logic_vector(7 downto 0);
        led     : out std_logic_vector(7 downto 0);
        an      : out std_logic_vector(3 downto 0);
        ssg     : out std_logic_vector(7 downto 0);
		  hs		 : out std_logic;
		  vs		 : out std_logic;
		  red, grn, blu : out std_logic;
		  LoopOut : out std_logic_vector(8 downto 0); 
		  LoopIn : in std_logic_vector(8 downto 0);

		  SS		: in	std_logic;
		  MOSI	: in std_logic;
		  MISO	: out std_logic;
		  SCK		: in std_logic;

		  TMSEN	: inout std_logic);


end BasysDemo;

architecture Behavioral of BasysDemo is

	------------------------------------------------------------------------
	-- Component Declarations
	------------------------------------------------------------------------
 	component vgaController is
    	Port ( mclk : in std_logic;
           hs : out std_logic;
           vs : out std_logic;
           red : out std_logic;
           grn : out std_logic;
           blu : out std_logic);
	end component;

 	component loopback is
    	Port ( mclk : in std_logic;
           rst : in std_logic;
           strt : in std_logic;
           pass : out std_logic;
           fail : out std_logic;
			  TstOut:  out std_logic_vector(8 downto 0);
			  TstIn:		in std_logic_vector(8 downto 0));
	end component;

	component SpiCtrl is
		Port ( mclk: in std_logic;
				rst: 	in std_logic;
			   pinSS : in std_logic;
			   pinSCK : in std_logic;
			   pinMOSI : in std_logic;
			   pinMISO : out std_logic;
				pinTMSEN : in std_logic);
	end component;


	------------------------------------------------------------------------
	-- Signal Declarations
	------------------------------------------------------------------------

	signal clkdiv  	: std_logic_vector(23 downto 0);
	signal cntr    	: std_logic_vector(3 downto 0);
	signal cclk    	: std_logic;

	signal dig    		: std_logic_vector(6 downto 0);
	signal passDisp   : std_logic_vector(6 downto 0);
	signal failDisp   : std_logic_vector(6 downto 0);
	signal spiDisp   : std_logic_vector(6 downto 0);

	signal tstctr     : std_logic_vector(6 downto 0);
	signal ssegkb		: std_logic_vector(6 downto 0);
	signal ankb			: std_logic_vector(3 downto 0);
	signal rst			: std_logic;
	signal tclk			: std_logic;

	signal digit	: std_logic_vector (1 downto 0);

	signal rstSpi			: std_logic := '0';
	signal rstLoopback	: std_logic := '0';
	signal strt 			: std_logic := '0';
	signal pass 			: std_logic;
	signal fail				: std_logic;
	signal spien			: std_logic := '0';


	------------------------------------------------------------------------
	-- Module Implementation
	------------------------------------------------------------------------

	begin

	vga1: vgaController port map (mclk => mclk, 
	 											hs => hs, 
												vs => vs, 
												red => red, 
												grn => grn, 
												blu => blu);
					 
 	loopback1: loopback port map (mclk => tclk,
												rst => rstLoopback,
												strt => strt,
												pass => pass,
												fail => fail,
												TstOut => LoopOut,
												TstIn => LoopIn);


 	spictrl1: spictrl port map (mclk => tclk,
												rst => rstSpi,
												pinSS => SS,
												pinSCK => SCK,
												pinMOSI => MOSI,
												pinMISO => MISO,
												pinTMSEN => TMSEN);

	rst <= btn(0);
	
	led(7 downto 1) <= swt(7 downto 1);
	led(0) <= swt(0);
	
	
	--7 SEGMENT DISPLAY ----------------------------------------

	dig <= passDisp when pass = '1' else 
			failDisp when fail = '1' else
			spiDisp when spien = '1' else
			tstctr;

	tstctr <=    "0111111" when cntr = "0000" else 
			"0000110" when cntr = "0001" else
			"1011011" when cntr = "0010" else
			"1001111" when cntr = "0011" else
			"1100110" when cntr = "0100" else
			"1101101" when cntr = "0101" else
			"1111101" when cntr = "0110" else
			"0000111" when cntr = "0111" else
			"1111111" when cntr = "1000" else
			"1101111" when cntr = "1001" else
			"0000000";

	an(0) <= '0' when digit = "00" and ((btn(3) = '0') or (pass = '1' or fail = '1') or (spien = '1')) else '1';
	an(1) <= '0' when digit = "01" and ((btn(2) = '0') or (pass = '1' or fail = '1') or (spien = '1')) else '1';
	an(2) <= '0' when digit = "10" and ((btn(1) = '0') or (pass = '1' or fail = '1') or (spien = '1')) else '1';
	an(3) <= '0' when digit = "11" and ((btn(0) = '0') or (pass = '1' or fail = '1') or (spien = '1')) else '1';

	--an <= btn when swt(0) = '0' when pass = '0' and fail = '0';


	ssg(6 downto 0) <= not dig;
	ssg(7) <= '1';

	


	passDisp <= "1110011" when digit = "00" else		--P
			  "1110111" when digit = "01" else			--A
			  "1101101" when digit = "10" else		 	--S
			  "1101101" when digit = "11" else			--S
			  "0000000";

	failDisp <= "1110001" when digit = "00" else	  	--F
			  "1110111" when digit = "01" else			--A
			  "0110000" when digit = "10" else			--I
			  "0111000" when digit = "11" else			--L
			  "0000000";

	spiDisp <= "1101101" when digit = "00" else		--S
			  "1110011" when digit = "01" else			--P
			  "0110000" when digit = "10" else			--I
			  "0000000";

	--counter used to display digits in 7Seg display							
	digit(0) <= 	clkdiv(18);
	digit(1) <=	   clkdiv(17);

	--trigger loopback test start
	strt <= '1' when btn(0) = '1'  and btn(3) = '1' and swt(0) = '0' else
	'0';

	--trigger loopback test reset
	rstLoopback	 <= '1' when btn(1) = '1'  and btn(2) = '1' and swt(0) = '0' else
		'0';

	--trigger SPI test
	spien <= '1' when btn(0) = '1' and btn(3) = '1' and swt(0) = '1' else
		'0' when btn(1) = '1'  and btn(2) = '1';

	TMSEN <= '1' when spien = '1' else
				'0';

	-- Divide the master clock (50Mhz) down to a lower frequency.
	process (mclk)
		begin
			if mclk = '1' and mclk'Event then
				clkdiv <= clkdiv + 1;
			end if;
		end process;

	cclk <= clkdiv(23);
	tclk <= clkdiv(9);

	process (cclk)
		begin
			if cclk = '1' and cclk'Event then
				if cntr = "1001" then
				    cntr <= "0000";
				else
				    cntr <= cntr + 1;
				end if;
			end if;
			
		end process;

end Behavioral;

